<?php

declare(strict_types=1);

namespace Drupal\language_roles\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\user\Entity\Role;

/**
 * Defines the 'language_role' field type.
 *
 * @FieldType(
 *   id = "language_role",
 *   label = @Translation("Language and role"),
 *   description = @Translation("A field type to store a combination language and role."),
 *   default_widget = "language_role",
 *   default_formatter = "language_role",
 * )
 */
final class LanguageRoleItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public function isEmpty(): bool {
    return !$this->get('language')->getValue() || !$this->get('role')->getValue();
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition): array {

    $properties['language'] = DataDefinition::create('string')
      ->setLabel(t('Language'));

    $properties['role'] = DataDefinition::create('string')
      ->setLabel(t('Role'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition): array {

    $columns = [
      'language' => [
        'type' => 'varchar',
        'not null' => TRUE,
        'description' => 'The language.',
        'length' => 32,
      ],
      'role' => [
        'type' => 'varchar',
        'not null' => TRUE,
        'description' => 'The role.',
        'length' => 255,
      ],
    ];

    $schema = [
      'columns' => $columns,
      'indexes' => [
        'language' => ['language'],
        'role' => ['role'],
      ],
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition): array {
    $language_manager = \Drupal::languageManager();
    $languages = $language_manager->getLanguages();
    $values['language'] = array_rand($languages);
    $role_storage = \Drupal::entityTypeManager()->getStorage('user_role');
    $roles = $role_storage->getQuery()
      ->condition('id', [Role::ANONYMOUS_ID, Role::AUTHENTICATED_ID], 'NOT IN')
      ->execute();
    $values['role'] = !empty($roles) ? array_rand($roles) : NULL;
    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public static function mainPropertyName() {
    return NULL;
  }

}
