<?php

declare(strict_types=1);

namespace Drupal\language_roles\Plugin\Field\FieldWidget;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\user\Entity\Role;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the 'language_role' field widget.
 *
 * @FieldWidget(
 *   id = "language_role",
 *   label = @Translation("Language and role"),
 *   field_types = {"language_role"},
 * )
 */
final class LanguageRoleWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs a new LanguageRoleWidget object.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly LanguageManagerInterface $languageManager,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $configuration['field_definition'], $configuration['settings'], $configuration['third_party_settings']);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('language_manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $languages = $this->languageManager->getLanguages();
    $language_options = [];
    foreach ($languages as $language) {
      $language_options[$language->getId()] = $language->getName();
    }
    $element['language'] = [
      '#type' => 'select',
      '#title' => $this->t('Language'),
      '#options' => $language_options,
      '#empty_value' => '',
      '#default_value' => $items[$delta]->language ?? NULL,
    ];
    $role_storage = $this->entityTypeManager->getStorage('user_role');
    $role_options = [];
    foreach ($role_storage->loadMultiple() as $role) {
      if (!in_array($role->id(), [Role::ANONYMOUS_ID, Role::AUTHENTICATED_ID])) {
        $role_options[$role->id()] = $role->label();
      }
    }
    $element['role'] = [
      '#type' => 'select',
      '#title' => $this->t('Role'),
      '#options' => $role_options,
      '#empty_value' => '',
      '#default_value' => $items[$delta]->role ?? NULL,
    ];
    return $element;
  }

}
