<?php

declare(strict_types=1);

namespace Drupal\Tests\language_roles\Functional;

use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\Tests\BrowserTestBase;

/**
 * Test access results related to language roles.
 *
 * @group language_roles
 */
final class LanguageRolesAccessTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'claro';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'language_roles',
    'content_translation',
    'node',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Enable the French and Spanish languages.
    ConfigurableLanguage::create(['label' => 'French', 'id' => 'fr'])->save();
    ConfigurableLanguage::create(['label' => 'Spanish', 'id' => 'es'])->save();

    // Create a content type.
    $this->drupalCreateContentType(['type' => 'page']);

    // Enable translation on pages.
    \Drupal::service('content_translation.manager')->setEnabled('node', 'page', TRUE);

    // Create a role with some access permissions.
    $this->drupalCreateRole([
      'create page content',
      'translate any entity',
    ], 'editor', 'Editor');

  }

  /**
   * Test that a user with a language role can only act in that language.
   */
  public function testLanguageRolePermissions(): void {
    $user = $this->drupalCreateUser();
    $user->set('language_roles', ['language' => 'fr', 'role' => 'editor']);
    $user->save();
    $this->drupalLogin($user);

    // Attempt to create a page in English.
    $this->drupalGet('node/add/page');
    $this->assertSession()->statusCodeEquals(403);

    // Attempt to create a page in French.
    $this->drupalGet('fr/node/add/page');
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Test the translate_target setting.
   */
  public function testTranslationTarget(): void {
    $node = $this->drupalCreateNode(['title' => 'English', 'type' => 'page', 'language' => 'en']);

    $user = $this->drupalCreateUser();
    $user->set('language_roles', [
      ['language' => 'fr', 'role' => 'editor'],
      ['language' => 'es', 'role' => 'editor'],
    ]);
    $user->save();
    $this->drupalLogin($user);

    // Attempt to create translation in non-matching language.
    $this->drupalGet('fr/node/' . $node->id() . '/translations/add/en/es');
    $this->assertSession()->statusCodeEquals(403);

    // Attempt to create translation in matching languages.
    $this->drupalGet('fr/node/' . $node->id() . '/translations/add/en/fr');
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet('es/node/' . $node->id() . '/translations/add/en/es');
    $this->assertSession()->statusCodeEquals(200);

    // Turn off enforcing of translation in language.
    \Drupal::configFactory()->getEditable('language_roles.settings')->set('translate_target', FALSE)->save();

    // Non-matching language should now be available.
    $this->drupalGet('fr/node/' . $node->id() . '/translations/add/en/es');
    $this->assertSession()->statusCodeEquals(200);

  }

}
